#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import gzip
import os
import shutil
from multiprocessing import Process

import click
import pymysql
import tqdm


def gunzip(input: str, output: str):
    pbar = tqdm.tqdm(total=os.path.getsize(input), unit='b', unit_scale=True, unit_divisor=1024)
    with open(input, 'rb') as inf, gzip.open(inf, "rb") as rf, open(output, 'wb') as of:
        while True:
            buf = rf.read(shutil.COPY_BUFSIZE)
            if not buf:
                break
            of.write(buf)
            pbar.n = inf.tell()
            pbar.update(0)


@click.command
@click.option("-p", "--password", help="password to use", required="True")
@click.option("-u", "--user", help="user to use", required="True")
@click.option("-h", "--host", help="database host", required="True")
@click.option("-d", "--database", help="database to use", required="True")
@click.option("-t", "--table", help="table name", required="True")
@click.option("-i", "--input", help="input CSV/TSV (gz) filename", required="True", type=click.Path(exists=True))
@click.option("-f", "--format" , help="SQL CSV/TSV format definition (default \"LINES TERMINATED BY '\\r\\n' IGNORE 1 ROWS\")", default="LINES TERMINATED BY '\\r\\n' IGNORE 1 ROWS")
def load_db(host: str, user: str, password: str, database: str, table: str, input: str, format: str):
    """Load a CSV/TSV (gz) file into MariaDB using LOAD DATA LOCAL INFILE"""
    with pymysql.connect(host=host, user=user, password=password, database=database, charset='utf8mb4', local_infile=True,
                         autocommit=True) as con, con.cursor() as cur:
        if input.endswith(".gz"):
            os.mkfifo("pipe.tsv", 0o600)
            p = Process(target=gunzip, args=(input, "pipe.tsv"))
            p.start()
            try:
                cur.execute(f"LOAD DATA LOCAL INFILE 'pipe.tsv' INTO TABLE {table} {format}")
                print(f"Ingested {cur.rowcount} rows.")
                cur.execute("SHOW WARNINGS")
                print(f"Errors and warnings encountered: {cur.fetchall()}")
                p.join()
            finally:
                p.terminate()
                os.remove("pipe.tsv")
        else:
            cur.execute(f"LOAD DATA LOCAL INFILE '{input}' INTO TABLE {table} {format}")
            print(f"Ingested {cur.rowcount} rows.")
            cur.execute("SHOW WARNINGS")
            print(f"Errors and warnings encountered: {cur.fetchall()}")


if __name__ == '__main__':
    load_db()





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from io import StringIO

import click
import pymysql


@click.command
@click.option("-p", "--password", help="password to use", required="True")
@click.option("-u", "--user", help="user to use", required="True")
@click.option("-h", "--host", help="database host", required="True")
@click.option("-d", "--database", help="database to use")
@click.option("-s", "--sql", help="input SQL filename", required="True", type=click.File(mode='rt'))
def run_sql(host: str, user: str, password: str, database: str, sql: StringIO):
    """Run a given SQL script file"""
    with pymysql.connect(host=host, user=user, password=password, charset='utf8mb4', local_infile=True,
                         autocommit=True, database=database) as con, con.cursor() as cur:
        for statement in sql.read().split(";"):
            if len(statement.strip()) > 0:
                print(f"Running statement:\n{statement}")
                cur.execute(statement)
                warnings = cur.fetchall()
                if len(warnings) > 0:
                    print(f"Errors and warnings encountered: {warnings} while running {statement}.")


if __name__ == '__main__':
    run_sql()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mdbutil']

package_data = \
{'': ['*']}

install_requires = \
['PyMySQL>=1.0.2,<2.0.0', 'click>=8.1.3,<9.0.0', 'tqdm>=4.64.0,<5.0.0']

entry_points = \
{'console_scripts': ['load-db = mdbutil.loaddb:load_db',
                     'run-sql = mdbutil.runsql:run_sql']}

setup_kwargs = {
    'name': 'mdbutil',
    'version': '0.1.3',
    'description': 'Utilities to run SQL and load a CSV/TSV (gz) file into MariaDB',
    'long_description': None,
    'author': 'Eetu Mäkelä',
    'author_email': 'eetu.makela@iki.fi',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
